#ifndef _CMD_HANDLING_H
#define _CMD_HANDLING_H

#define DIMRANGE 12
#define CMD_BLKSIZE 64   /* How many commands should we allocate space for at a time? */
#define DEV_BLKSIZE 5    /*  How many devices in a command allocated at a time */
#define ISDIMCMD(cmd) ((cmd == DIM) || (cmd == BRIGHT))
#define CMDHASDEVS(cmd) ((cmd == ON) || (cmd == OFF))  /* command need device to work on? */
#define HOUSENAME(house) (((house < 0) || (house > 15)) ? \
          '?':"ABCDEFGHIJKLMNOP"[house])
#define DEVNAME(dev) (((dev < 0) || (dev > 16)) ? 0 : dev + 1)

typedef struct {
    int numdevs;
    int allocateddevs;
    int *devs;
} br_device_list;

typedef struct {
    int inverse;
    int repeat;
    int fd;
    int numcmds;
    int allocatedcmds;
    br_device_list **devs;
    char *houses;
    int *dimlevels;
    int *cmds;
} br_control_info;

void br_error(br_control_info *, char *);
int br_inverse_cmd(int /* command */);
int br_set_fd(br_control_info *, int /* file descriptor */);
int br_execute(br_control_info *);
br_device_list *br_new_device_list();
int br_free_device_list(br_device_list *);
int br_add_device(br_device_list *, int /* device */);
int br_del_device(br_device_list *, int /* device */);
int br_malloc_cmds(br_control_info *, int /* number of commands */);
int br_realloc_cmds(br_control_info *, int /* number of commands */);
int br_free_cmds(br_control_info *);
int br_add_dl_cmd(br_control_info *, int /* command */, int /* house */,
               br_device_list * /* devices */, int /* dim level */);
int br_add_cmd(br_control_info *, int /* command */, int /* house */,
               int /* device */, int /* dim level */);
int br_del_cmd(br_control_info *, int /* command index */);
br_control_info *br_new_control_info();
int br_free_control_info(br_control_info *);
int br_strtodl(char * /* dlptr */, br_device_list * /* devices */, char ** /* endptr */);
int br_dlcat(br_device_list * /* devices a */, br_device_list * /* devices b */);
int br_strtohc(char * /* hcptr */, char ** /* endptr */);
int br_get_next_device(br_device_list * /* devices */, int * /* index */);
int br_get_num_devices(br_device_list * /* devices */);

#endif