#ifdef __cplusplus
extern "C" {
#endif

#include <string.h>
#include "EXTERN.h"
#include "perl.h"
#include "XSUB.h"

#include "br_cmd.h"
#include "br_cmd_engine.h"

#ifdef __cplusplus
}
#endif


/*
 *
 * BottleRocket.xs -- Perl module to use BottleRocket commands
 *  (c) 1999 Tymm Twillman.  LGPL applies.  No warranties.
 *  Not for use in life-saving devices.  This stuff won't get
 *  you dates.  Trust me.
 */

int br_getcmd(char *arg)
{
/*
 * Convert a BottleRocket command to the appropriate token
 */

    if (!strcasecmp(arg, "ON"))
        return ON;

    if (!strcasecmp(arg, "OFF"))
        return OFF;

    if (!strcasecmp(arg, "DIM"))
        return DIM;

    if (!strcasecmp(arg, "BRIGHT"))
        return BRIGHT;

    if (!strcasecmp(arg, "ALL_ON"))
        return ALL_ON;

    if (!strcasecmp(arg, "ALL_OFF"))
        return ALL_OFF;

    if (!strcasecmp(arg, "LAMPS_ON"))
        return ALL_LAMPS_ON;

    if (!strcasecmp(arg, "LAMPS_OFF"))
        return ALL_LAMPS_OFF;

    return -1;
}

void my_error_handler(char *where, char *problem)
{
    croak("where->%s  problem->%s");
}


MODULE = BottleRocket		PACKAGE = BottleRocket		

void
x10_cmd(FileHandle, Unit, Command, ...)
	FILE *FileHandle
	char *Unit
	char *Command

	PREINIT:
	va_list ap;
	register int i;
	char unit_out;
	int cmd_out;
	int fd;
	char *end;
	br_device_list *devs;
	int housecode;
	int dev;
	int dimlevel;
	int idx = 0;

	CODE:

	cmd_out = br_getcmd(Command);
		
	if (cmd_out < 0) {
	    croak("Invalid command");
	    return;
	}

	if (ISDIMCMD(cmd_out)) {
        dimlevel = SvIV(ST(3));
    }

	br_error_handler = my_error_handler;
	
	if ((FileHandle == NULL) || ((fd = fileno(FileHandle)) < 0)) {
	    croak("Invalid filehandle");
	    return;
	}
    		
	housecode = br_strtohc(Unit, &end);
	
	if (housecode < 0) {
	    croak("Invalid device list");
	    return;
	}

	devs = br_new_device_list();
		
	if (*end) {
        br_strtodl(end, devs, &end);
        if (*end != '\0') {
            croak("Invalid device list");
            return;
        }
	} else if ((cmd_out == ON) || (cmd_out == OFF)) {
	    croak("No devices specified for command that needs them");
	    return;
	}

	if (br_get_num_devices(devs)) {
	    while ((dev = br_get_next_device(devs, &idx)) >= 0) {
	        if (ISDIMCMD(cmd_out)) {
	            x10_br_out(fd, dev, ON);
	            for (i = 0; i < dimlevel; i++)
	                x10_br_out(fd, 0, cmd_out);
	        } else {
	            x10_br_out(fd, dev, cmd_out);
	        }
	    }
	} else {
	    x10_br_out(fd, 0, cmd_out);
	}
	
	br_free_device_list(devs);

	
		
	return;
